<?php

namespace Drupal\ww_publish;

use Drupal\Core\File\FileSystemInterface;
use Drupal\media\Entity\Media;
use Drupal\media\Entity\MediaType;

class Image {

  /**
   * WoodWing Studio image ID.
   *
   * @var integer
   */
  private $wwId;

  /**
   * @var \Drupal\ww_publish\Message
   */
  private $message;

  /**
   * WoodWing Studio article ID field.
   *
   * @var string
   */
  private $idField;

  /**
   * Configuration of the ww_publish module.
   *
   * @var \Drupal\Core\Config\Config
   */
  private $config;

  /**
   * A logger instance.
   *
   * @var \Psr\Log\LoggerInterface
   */
  private $logger;

  /**
   * Constructor.
   *
   * @param integer $ww_id
   *   WoodWing Studio image ID.
   * @param \Drupal\ww_publish\Message $message
   *   Article metadata.
   * @param \Drupal\Core\Config\Config $config
   *   Configuration of the ww_publish module.
   * @param \Psr\Log\LoggerInterface $logger
   *   A logger instance.
   */
  public function __construct($ww_id, Message $message, $config, $logger) {
    $this->wwId = $ww_id;
    $this->message = $message;

    // This class is currently only called if the image media type exists.
    // @todo Make more flexible, support any media type name that has this
    //   configured.
    $image = MediaType::load('image');
    $this->idField = $image->getThirdPartySetting('ww_publish', 'id_field');
    $this->config = $config;
    $this->logger = $logger;
  }

  /**
   * Get Media entity of the image.
   *
   * @return \Drupal\Core\Entity\EntityInterface
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function getMediaEntity() {

    if (!$this->idField) {
      throw new \Exception('ID field media type third party setting not configured');
    }

    // Find the media entity.
    $imageIds = \Drupal::entityQuery('media')
      ->condition('bundle','image')
      ->condition($this->idField, $this->wwId)
      ->execute();

    if ($this->config->get('debug_mode'))
      $this->logger->debug('Media ID Field: @id_field, WW ID: @ww_id, image IDs: <pre><code>@image_ids</code></pre>', ['@id_field' => $this->idField, '@ww_id' => $this->wwId, '@image_ids' => print_r($imageIds, TRUE)]);

    if (empty($imageIds)) {
      return $this->createMediaEntity();
    } else {
      return Media::load(reset($imageIds));
    }
  }

  /**
   * Create a new image media entity.
   *
   * @return \Drupal\Core\Entity\EntityInterface
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  private function createMediaEntity() {
    if ($this->message->getImage($this->wwId)) {
      $filePath = $this->message->getImage($this->wwId);
    } else {
      return NULL;
    }
    $file_data = file_get_contents($filePath);

    // Create the directory if it does not exist.
    $directory_path = 'public://' . $this->config->get('ww_path') . '/' . date('Y-m');
    $fileSystem = \Drupal::service('file_system');
    $fileSystem->prepareDirectory($directory_path, FileSystemInterface::CREATE_DIRECTORY);

    $destination = $directory_path . '/' .  $this->message->getArticleMetadata()->getFileName($this->wwId, $filePath);
    if ($this->config->get('debug_mode'))
      $this->logger->debug('File destination: <pre><code>@destination</code></pre>', ['@destination' => print_r($destination, TRUE)]);

    $file = file_save_data($file_data, $destination);

    $mediaData = [
      'bundle' => 'image',
      'name' => $file->getFilename(),
      'status' => 1,
      'uid' => $this->message->getArticleMetadata()->getAuthor() ?: 0,
      'field_media_image' => [
        'target_id' => $file->id(),
        'alt' => $file->getFilename(),
        'title' => $file->getFilename(),
      ],
      $this->idField => $this->wwId,
    ];
    if ($this->config->get('debug_mode'))
      $this->logger->debug('Media data: <pre><code>@media_data</code></pre>', ['@media_data' => print_r($mediaData, TRUE)]);
    $media = Media::create($mediaData);
    $media->save();

    return $media;
  }

}
